@@ -1,24 +1,24 @@
-
 import itertools
 
 import numpy
 
 from brian2.codegen.cpp_prefs import C99Check
-from brian2.utils.stringtools import (deindent, stripped_deindented_lines,
-                                      word_substitute)
-from brian2.utils.logger import get_logger
-from brian2.parsing.rendering import CPPNodeRenderer
-from brian2.core.functions import Function, DEFAULT_FUNCTIONS
-from brian2.core.preferences import prefs, BrianPreference
+from brian2.core.functions import DEFAULT_FUNCTIONS, Function
+from brian2.core.preferences import BrianPreference, prefs
 from brian2.core.variables import ArrayVariable
+from brian2.parsing.rendering import CPPNodeRenderer
+from brian2.utils.logger import get_logger
+from brian2.utils.stringtools import (
+    deindent,
+    stripped_deindented_lines,
+    word_substitute,
+)
 
 from .base import CodeGenerator
 
 logger = get_logger(__name__)
 
-__all__ = ['CPPCodeGenerator',
-           'c_data_type'
-           ]
+__all__ = ["CPPCodeGenerator", "c_data_type"]
 
 
 def c_data_type(dtype):
@@ -31,28 +31,28 @@ def c_data_type(dtype):
     if dtype is int:
         dtype = numpy.array([1]).dtype.type
     if dtype is float:
-        dtype = numpy.array([1.]).dtype.type
+        dtype = numpy.array([1.0]).dtype.type
 
     if dtype == numpy.float32:
-        dtype = 'float'
+        dtype = "float"
     elif dtype == numpy.float64:
-        dtype = 'double'
+        dtype = "double"
     elif dtype == numpy.int8:
-        dtype = 'int8_t'
+        dtype = "int8_t"
     elif dtype == numpy.int16:
-        dtype = 'int16_t'
+        dtype = "int16_t"
     elif dtype == numpy.int32:
-        dtype = 'int32_t'
+        dtype = "int32_t"
     elif dtype == numpy.int64:
-        dtype = 'int64_t'
+        dtype = "int64_t"
     elif dtype == numpy.uint16:
-        dtype = 'uint16_t'
+        dtype = "uint16_t"
     elif dtype == numpy.uint32:
-        dtype = 'uint32_t'
+        dtype = "uint32_t"
     elif dtype == numpy.uint64:
-        dtype = 'uint64_t'
+        dtype = "uint64_t"
     elif dtype == numpy.bool_ or dtype is bool:
-        dtype = 'char'
+        dtype = "char"
     else:
         raise ValueError(f"dtype {str(dtype)} not known.")
     return dtype
@@ -60,22 +60,22 @@ def c_data_type(dtype):
 
 # Preferences
 prefs.register_preferences(
-    'codegen.generators.cpp',
-    'C++ codegen preferences',
-    restrict_keyword = BrianPreference(
-        default='__restrict',
+    "codegen.generators.cpp",
+    "C++ codegen preferences",
+    restrict_keyword=BrianPreference(
+        default="__restrict",
         docs="""
         The keyword used for the given compiler to declare pointers as restricted.
-        
+
         This keyword is different on different compilers, the default works for
         gcc and MSVS.
         """,
-        ),
-    flush_denormals = BrianPreference(
+    ),
+    flush_denormals=BrianPreference(
         default=False,
         docs="""
         Adds code to flush denormals to zero.
-        
+
         The code is gcc and architecture specific, so may not compile on all
         platforms. The code, for reference is::
 
@@ -83,16 +83,16 @@ prefs.register_preferences(
             unsigned csr = __builtin_ia32_stmxcsr();
             csr |= CSR_FLUSH_TO_ZERO;
             __builtin_ia32_ldmxcsr(csr);
-            
+
         Found at `<http://stackoverflow.com/questions/2487653/avoiding-denormal-values-in-c>`_.
         """,
-        ),
-    )
+    ),
+)
 
 
-typestrs = ['int', 'long', 'long long', 'float', 'double', 'long double']
-floattypestrs = ['float', 'double', 'long double']
-hightype_support_code = 'template < typename T1, typename T2 > struct _higher_type;\n'
+typestrs = ["int", "long", "long long", "float", "double", "long double"]
+floattypestrs = ["float", "double", "long double"]
+hightype_support_code = "template < typename T1, typename T2 > struct _higher_type;\n"
 for ix, xtype in enumerate(typestrs):
     for iy, ytype in enumerate(typestrs):
         hightype = typestrs[max(ix, iy)]
@@ -128,38 +128,39 @@ pow_support_code = """
 #endif
 """
 
-_universal_support_code = (hightype_support_code + mod_support_code +
-                           floordiv_support_code + pow_support_code)
+_universal_support_code = (
+    hightype_support_code + mod_support_code + floordiv_support_code + pow_support_code
+)
 
 
 class CPPCodeGenerator(CodeGenerator):
     """
     C++ language
-    
+
     C++ code templates should provide Jinja2 macros with the following names:
-    
+
     ``main``
         The main loop.
     ``support_code``
         The support code (function definitions, etc.), compiled in a separate
         file.
-        
+
     For user-defined functions, there are two keys to provide:
-    
+
     ``support_code``
         The function definition which will be added to the support code.
     ``hashdefine_code``
         The ``#define`` code added to the main loop.
-        
+
     See `TimedArray` for an example of these keys.
     """
 
-    class_name = 'cpp'
+    class_name = "cpp"
 
     universal_support_code = _universal_support_code
 
     def __init__(self, *args, **kwds):
-        super(CPPCodeGenerator, self).__init__(*args, **kwds)
+        super().__init__(*args, **kwds)
         self.c_data_type = c_data_type
 
     @property
@@ -168,12 +169,13 @@ class CPPCodeGenerator(CodeGenerator):
 
     @property
     def flush_denormals(self):
-        return prefs['codegen.generators.cpp.flush_denormals']
+        return prefs["codegen.generators.cpp.flush_denormals"]
 
     @staticmethod
     def get_array_name(var, access_data=True):
         # We have to do the import here to avoid circular import dependencies.
         from brian2.devices.device import get_device
+
         device = get_device()
         if access_data:
             return f"_ptr{device.get_array_name(var)}"
@@ -182,23 +184,33 @@ class CPPCodeGenerator(CodeGenerator):
 
     def translate_expression(self, expr):
         expr = word_substitute(expr, self.func_name_replacements)
-        return CPPNodeRenderer(auto_vectorise=self.auto_vectorise).render_expr(expr).strip()
+        return (
+            CPPNodeRenderer(auto_vectorise=self.auto_vectorise)
+            .render_expr(expr)
+            .strip()
+        )
 
     def translate_statement(self, statement):
-        var, op, expr, comment = (statement.var, statement.op,
-                                  statement.expr, statement.comment)
+        var, op, expr, comment = (
+            statement.var,
+            statement.op,
+            statement.expr,
+            statement.comment,
+        )
         # For C++ we replace complex expressions involving boolean variables into a sequence of
         # if/then expressions with simpler expressions. This is provided by the optimise_statements
         # function.
-        if statement.used_boolean_variables is not None and len(statement.used_boolean_variables):
+        if statement.used_boolean_variables is not None and len(
+            statement.used_boolean_variables
+        ):
             used_boolvars = statement.used_boolean_variables
             bool_simp = statement.boolean_simplified_expressions
-            if op == ':=':
+            if op == ":=":
                 # we have to declare the variable outside the if/then statement (which
                 # unfortunately means we can't make it const but the optimisation is worth
                 # it anyway).
                 codelines = [f"{self.c_data_type(statement.dtype)} {var};"]
-                op = '='
+                op = "="
             else:
                 codelines = []
             firstline = True
@@ -213,30 +225,30 @@ class CPPCodeGenerator(CodeGenerator):
                     else:
                         atomics.append(f"!{boolvar}")
                 if firstline:
-                    line = ''
+                    line = ""
                 else:
-                    line = 'else '
+                    line = "else "
                 # only need another if statement when we have more than one boolean variables
-                if firstline or len(used_boolvars)>1:
+                if firstline or len(used_boolvars) > 1:
                     line += f"if({' && '.join(atomics)})"
-                line += '\n    '
+                line += "\n    "
                 line += f"{var} {op} {self.translate_expression(simp_expr)};"
                 codelines.append(line)
                 firstline = False
-            code = '\n'.join(codelines)
+            code = "\n".join(codelines)
         else:
-            if op == ':=':
+            if op == ":=":
                 decl = f"{self.c_data_type(statement.dtype)} "
-                op = '='
+                op = "="
                 if statement.constant:
                     decl = f"const {decl}"
             else:
-                decl = ''
+                decl = ""
             code = f"{decl + var} {op} {self.translate_expression(expr)};"
         if len(comment):
             code += f" // {comment}"
         return code
-    
+
     def translate_to_read_arrays(self, read, write, indices):
         lines = []
         # index and read arrays (index arrays first)
@@ -244,9 +256,9 @@ class CPPCodeGenerator(CodeGenerator):
             index_var = self.variable_indices[varname]
             var = self.variables[varname]
             if varname not in write:
-                line = 'const '
+                line = "const "
             else:
-                line = ''
+                line = ""
             line = f"{line + self.c_data_type(var.dtype)} {varname} = "
             line = f"{line + self.get_array_name(var)}[{index_var}];"
             lines.append(line)
@@ -269,7 +281,7 @@ class CPPCodeGenerator(CodeGenerator):
             line = self.translate_statement(stmt)
             if stmt.var in conditional_write_vars:
                 condvar = conditional_write_vars[stmt.var]
-                lines.append(f'if({condvar})')
+                lines.append(f"if({condvar})")
                 lines.append(f"    {line}")
             else:
                 lines.append(line)
@@ -300,7 +312,7 @@ class CPPCodeGenerator(CodeGenerator):
         lines += self.translate_to_statements(statements, cond_write)
         # write arrays
         lines += self.translate_to_write_arrays(write)
-        return stripped_deindented_lines('\n'.join(lines))
+        return stripped_deindented_lines("\n".join(lines))
 
     def translate_statement_sequence(self, sc_statements, ve_statements):
         # This function is overwritten, since we do not want to completely
@@ -316,10 +328,12 @@ class CPPCodeGenerator(CodeGenerator):
         for block_name in sc_statements:
             sc_block = sc_statements[block_name]
             ve_block = ve_statements[block_name]
-            (sc_read, sc_write,
-             sc_indices, sc_cond_write) = self.arrays_helper(sc_block)
-            (ve_read, ve_write,
-             ve_indices, ve_cond_write) = self.arrays_helper(ve_block)
+            (sc_read, sc_write, sc_indices, sc_cond_write) = self.arrays_helper(
+                sc_block
+            )
+            (ve_read, ve_write, ve_indices, ve_cond_write) = self.arrays_helper(
+                ve_block
+            )
             # We want to read all scalar variables that are needed in the
             # vector code already in the scalar code, if they are not written
             for varname in set(ve_read):
@@ -328,11 +342,10 @@ class CPPCodeGenerator(CodeGenerator):
                     sc_read.add(varname)
                     ve_read.remove(varname)
 
-            for (code, stmts, read, write, indices,
-                 cond_write) in [(sc_code, sc_block, sc_read, sc_write,
-                                  sc_indices, sc_cond_write),
-                                 (ve_code, ve_block, ve_read, ve_write,
-                                  ve_indices, ve_cond_write)]:
+            for code, stmts, read, write, indices, cond_write in [
+                (sc_code, sc_block, sc_read, sc_write, sc_indices, sc_cond_write),
+                (ve_code, ve_block, ve_read, ve_write, ve_indices, ve_cond_write),
+            ]:
                 lines = []
                 # index and read arrays (index arrays first)
                 lines += self.translate_to_read_arrays(read, write, indices)
@@ -342,11 +355,10 @@ class CPPCodeGenerator(CodeGenerator):
                 lines += self.translate_to_statements(stmts, cond_write)
                 # write arrays
                 lines += self.translate_to_write_arrays(write)
-                code[block_name] = stripped_deindented_lines('\n'.join(lines))
+                code[block_name] = stripped_deindented_lines("\n".join(lines))
 
         return sc_code, ve_code, kwds
 
-
     def denormals_to_zero_code(self):
         if self.flush_denormals:
             return """
@@ -356,7 +368,7 @@ class CPPCodeGenerator(CodeGenerator):
             __builtin_ia32_ldmxcsr(csr);
             """
         else:
-            return ''
+            return ""
 
     def _add_user_function(self, varname, variable, added):
         impl = variable.implementations[self.codeobj_class]
@@ -378,25 +390,26 @@ class CPPCodeGenerator(CodeGenerator):
                     dep_impl_name = dep.pyfunc.__name__
                 if dep_name != dep_impl_name:
                     funccode = word_substitute(funccode, {dep_name: dep_impl_name})
-            funccode = {'support_code': funccode}
+            funccode = {"support_code": funccode}
         if funccode is not None:
             # To make namespace variables available to functions, we
             # create global variables and assign to them in the main
             # code
             func_namespace = impl.get_namespace(self.owner) or {}
             for ns_key, ns_value in func_namespace.items():
-                if hasattr(ns_value, 'dtype'):
+                if hasattr(ns_value, "dtype"):
                     if ns_value.shape == ():
-                        raise NotImplementedError((
-                        'Directly replace scalar values in the function '
-                        'instead of providing them via the namespace'))
+                        raise NotImplementedError(
+                            "Directly replace scalar values in the function "
+                            "instead of providing them via the namespace"
+                        )
                     type_str = f"{self.c_data_type(ns_value.dtype)}*"
                 else:  # e.g. a function
-                    type_str = 'py::object'
-                support_code.append(f'static {type_str} _namespace{ns_key};')
-                pointers.append(f'_namespace{ns_key} = {ns_key};')
-            support_code.append(deindent(funccode.get('support_code', '')))
-            hash_defines.append(deindent(funccode.get('hashdefine_code', '')))
+                    type_str = "py::object"
+                support_code.append(f"static {type_str} _namespace{ns_key};")
+                pointers.append(f"_namespace{ns_key} = {ns_key};")
+            support_code.append(deindent(funccode.get("support_code", "")))
+            hash_defines.append(deindent(funccode.get("hashdefine_code", "")))
 
         dep_hash_defines = []
         dep_pointers = []
@@ -416,10 +429,12 @@ class CPPCodeGenerator(CodeGenerator):
                         dep_support_code.extend(sc)
                         user_functions.extend(uf)
 
-        return (dep_hash_defines + hash_defines,
-                dep_pointers + pointers,
-                dep_support_code + support_code,
-                user_functions)
+        return (
+            dep_hash_defines + hash_defines,
+            dep_pointers + pointers,
+            dep_support_code + support_code,
+            user_functions,
+        )
 
     def determine_keywords(self):
         # set up the restricted pointers, these are used so that the compiler
@@ -432,21 +447,25 @@ class CPPCodeGenerator(CodeGenerator):
         template_kwds = {}
         # Again, do the import here to avoid a circular dependency.
         from brian2.devices.device import get_device
+
         device = get_device()
-        for varname, var in self.variables.items():
+        for var in self.variables.values():
             if isinstance(var, ArrayVariable):
                 # This is the "true" array name, not the restricted pointer.
                 array_name = device.get_array_name(var)
                 pointer_name = self.get_array_name(var)
                 if pointer_name in handled_pointers:
                     continue
-                if getattr(var, 'ndim', 1) > 1:
+                if getattr(var, "ndim", 1) > 1:
                     continue  # multidimensional (dynamic) arrays have to be treated differently
                 restrict = self.restrict
                 # turn off restricted pointers for scalars for safety
                 if var.scalar or var.size == 1:
-                    restrict = ' '
-                line = f'{self.c_data_type(var.dtype)}* {restrict} {pointer_name} = {array_name};'
+                    restrict = " "
+                line = (
+                    f"{self.c_data_type(var.dtype)}* {restrict} {pointer_name} ="
+                    f" {array_name};"
+                )
                 pointers.append(line)
                 handled_pointers.add(pointer_name)
 
@@ -466,37 +485,56 @@ class CPPCodeGenerator(CodeGenerator):
                     hash_defines.extend(hd)
         support_code.append(self.universal_support_code)
 
-
-        keywords = {'pointers_lines': stripped_deindented_lines('\n'.join(pointers)),
-                    'support_code_lines': stripped_deindented_lines('\n'.join(support_code)),
-                    'hashdefine_lines': stripped_deindented_lines('\n'.join(hash_defines)),
-                    'denormals_code_lines': stripped_deindented_lines('\n'.join(self.denormals_to_zero_code())),
-                    }
+        keywords = {
+            "pointers_lines": stripped_deindented_lines("\n".join(pointers)),
+            "support_code_lines": stripped_deindented_lines("\n".join(support_code)),
+            "hashdefine_lines": stripped_deindented_lines("\n".join(hash_defines)),
+            "denormals_code_lines": stripped_deindented_lines(
+                "\n".join(self.denormals_to_zero_code())
+            ),
+        }
         keywords.update(template_kwds)
         return keywords
 
+
 ################################################################################
 # Implement functions
 ################################################################################
 # Functions that exist under the same name in C++
-for func in ['sin', 'cos', 'tan', 'sinh', 'cosh', 'tanh', 'exp', 'log',
-             'log10', 'sqrt', 'ceil', 'floor']:
-    DEFAULT_FUNCTIONS[func].implementations.add_implementation(CPPCodeGenerator,
-                                                               code=None)
-DEFAULT_FUNCTIONS['expm1'].implementations.add_implementation(CPPCodeGenerator,
-                                                              code=None,
-                                                              availability_check=C99Check('expm1'))
-DEFAULT_FUNCTIONS['log1p'].implementations.add_implementation(CPPCodeGenerator,
-                                                              code=None,
-                                                              availability_check=C99Check('log1p'))
+for func in [
+    "sin",
+    "cos",
+    "tan",
+    "sinh",
+    "cosh",
+    "tanh",
+    "exp",
+    "log",
+    "log10",
+    "sqrt",
+    "ceil",
+    "floor",
+]:
+    DEFAULT_FUNCTIONS[func].implementations.add_implementation(
+        CPPCodeGenerator, code=None
+    )
+DEFAULT_FUNCTIONS["expm1"].implementations.add_implementation(
+    CPPCodeGenerator, code=None, availability_check=C99Check("expm1")
+)
+DEFAULT_FUNCTIONS["log1p"].implementations.add_implementation(
+    CPPCodeGenerator, code=None, availability_check=C99Check("log1p")
+)
 
 # Functions that need a name translation
-for func, func_cpp in [('arcsin', 'asin'), ('arccos', 'acos'), ('arctan', 'atan'),
-                       ('int', 'int_')  # from stdint_compat.h
-                       ]:
-    DEFAULT_FUNCTIONS[func].implementations.add_implementation(CPPCodeGenerator,
-                                                               code=None,
-                                                               name=func_cpp)
+for func, func_cpp in [
+    ("arcsin", "asin"),
+    ("arccos", "acos"),
+    ("arctan", "atan"),
+    ("int", "int_"),  # from stdint_compat.h
+]:
+    DEFAULT_FUNCTIONS[func].implementations.add_implementation(
+        CPPCodeGenerator, code=None, name=func_cpp
+    )
 
 exprel_code = """
 static inline double _exprel(double x)
@@ -508,51 +546,53 @@ static inline double _exprel(double x)
     return expm1(x)/x;
 }
 """
-DEFAULT_FUNCTIONS['exprel'].implementations.add_implementation(CPPCodeGenerator,
-                                                               code=exprel_code,
-                                                               name='_exprel',
-                                                               availability_check=C99Check('exprel'))
+DEFAULT_FUNCTIONS["exprel"].implementations.add_implementation(
+    CPPCodeGenerator,
+    code=exprel_code,
+    name="_exprel",
+    availability_check=C99Check("exprel"),
+)
 
 abs_code = """
 #define _brian_abs std::abs
 """
-DEFAULT_FUNCTIONS['abs'].implementations.add_implementation(CPPCodeGenerator,
-                                                            code=abs_code,
-                                                            name='_brian_abs')
+DEFAULT_FUNCTIONS["abs"].implementations.add_implementation(
+    CPPCodeGenerator, code=abs_code, name="_brian_abs"
+)
 
 clip_code = """
         template <typename T>
         static inline T _clip(const T value, const double a_min, const double a_max)
         {
-	        if (value < a_min)
-	            return a_min;
-	        if (value > a_max)
-	            return a_max;
-	        return value;
-	    }
+            if (value < a_min)
+                return a_min;
+            if (value > a_max)
+                return a_max;
+            return value;
+        }
         """
-DEFAULT_FUNCTIONS['clip'].implementations.add_implementation(CPPCodeGenerator,
-                                                             code=clip_code,
-                                                             name='_clip')
+DEFAULT_FUNCTIONS["clip"].implementations.add_implementation(
+    CPPCodeGenerator, code=clip_code, name="_clip"
+)
 
 sign_code = """
         template <typename T> static inline int _sign(T val) {
             return (T(0) < val) - (val < T(0));
         }
         """
-DEFAULT_FUNCTIONS['sign'].implementations.add_implementation(CPPCodeGenerator,
-                                                             code=sign_code,
-                                                             name='_sign')
+DEFAULT_FUNCTIONS["sign"].implementations.add_implementation(
+    CPPCodeGenerator, code=sign_code, name="_sign"
+)
 
 timestep_code = """
 static inline int64_t _timestep(double t, double dt)
 {
-    return (int64_t)((t + 1e-3*dt)/dt); 
+    return (int64_t)((t + 1e-3*dt)/dt);
 }
 """
-DEFAULT_FUNCTIONS['timestep'].implementations.add_implementation(CPPCodeGenerator,
-                                                                 code=timestep_code,
-                                                                 name='_timestep')
+DEFAULT_FUNCTIONS["timestep"].implementations.add_implementation(
+    CPPCodeGenerator, code=timestep_code, name="_timestep"
+)
 
 poisson_code = """
 double _loggam(double x) {
@@ -641,7 +681,9 @@ int32_t _poisson(double lam, int32_t _idx) {
 }
 """
 
-DEFAULT_FUNCTIONS['poisson'].implementations.add_implementation(CPPCodeGenerator,
-                                                                code=poisson_code,
-                                                                name='_poisson',
-                                                                dependencies={'_rand': DEFAULT_FUNCTIONS['rand']})
\ No newline at end of file
+DEFAULT_FUNCTIONS["poisson"].implementations.add_implementation(
+    CPPCodeGenerator,
+    code=poisson_code,
+    name="_poisson",
+    dependencies={"_rand": DEFAULT_FUNCTIONS["rand"]},
+)
