@@ -3,32 +3,39 @@ Module containing the `Device` base class as well as the `RuntimeDevice`
 implementation and some helper functions to access/set devices.
 """
 
-from weakref import WeakKeyDictionary
 import numbers
+from weakref import WeakKeyDictionary
 
 import numpy as np
 
-from brian2.memory.dynamicarray import DynamicArray, DynamicArray1D
 from brian2.codegen.targets import codegen_targets
 from brian2.core.names import find_name
 from brian2.core.preferences import prefs
 from brian2.core.variables import ArrayVariable, DynamicArrayVariable
+from brian2.memory.dynamicarray import DynamicArray, DynamicArray1D
 from brian2.units import ms
 from brian2.utils.logger import get_logger
 from brian2.utils.stringtools import code_representation, indent
 
-__all__ = ['Device', 'RuntimeDevice',
-           'get_device', 'set_device',
-           'all_devices', 'reinit_devices', 'reinit_and_delete',
-           'reset_device', 'device', 'seed'
-           ]
+__all__ = [
+    "Device",
+    "RuntimeDevice",
+    "get_device",
+    "set_device",
+    "all_devices",
+    "reinit_devices",
+    "reinit_and_delete",
+    "reset_device",
+    "device",
+    "seed",
+]
 
 logger = get_logger(__name__)
 
 all_devices = {}
 
 
-prefs.register_preferences('devices', 'Device preferences')
+prefs.register_preferences("devices", "Device preferences")
 
 
 #: caches the automatically determined code generation target
@@ -48,34 +55,39 @@ def auto_target():
     """
     global _auto_target
     if _auto_target is None:
-        target_dict = dict((target.class_name, target)
-                           for target in codegen_targets
-                           if target.class_name)
+        target_dict = {
+            target.class_name: target for target in codegen_targets if target.class_name
+        }
         using_fallback = False
-        if 'cython' in target_dict and target_dict['cython'].is_available():
-            _auto_target = target_dict['cython']
+        if "cython" in target_dict and target_dict["cython"].is_available():
+            _auto_target = target_dict["cython"]
         else:
-            _auto_target = target_dict['numpy']
+            _auto_target = target_dict["numpy"]
             using_fallback = True
 
         if using_fallback:
-            logger.info('Cannot use compiled code, falling back to the numpy '
-                        'code generation target. Note that this will likely '
-                        'be slower than using compiled code. Set the code '
-                        'generation to numpy manually to avoid this message:\n'
-                        'prefs.codegen.target = "numpy"',
-                        'codegen_fallback', once=True)
+            logger.info(
+                "Cannot use compiled code, falling back to the numpy "
+                "code generation target. Note that this will likely "
+                "be slower than using compiled code. Set the code "
+                "generation to numpy manually to avoid this message:\n"
+                'prefs.codegen.target = "numpy"',
+                "codegen_fallback",
+                once=True,
+            )
         else:
-            logger.debug(('Chosing %r as the code generation '
-                         'target.') % _auto_target.class_name)
+            logger.debug(
+                "Chosing %r as the code generation target." % _auto_target.class_name
+            )
 
     return _auto_target
 
 
-class Device(object):
+class Device:
     """
     Base Device object.
     """
+
     def __init__(self):
         #: The network schedule that this device supports. If the device only
         #: supports a specific, fixed schedule, it has to set this attribute to
@@ -225,7 +237,7 @@ class Device(object):
         """
         raise NotImplementedError()
 
-    def code_object_class(self, codeobj_class=None, fallback_pref='codegen.target'):
+    def code_object_class(self, codeobj_class=None, fallback_pref="codegen.target"):
         """
         Return `CodeObject` class according to input/default settings
 
@@ -245,85 +257,117 @@ class Device(object):
             The `CodeObject` class that should be used
         """
         if isinstance(codeobj_class, str):
-            raise TypeError("codeobj_class argument given to code_object_class device method "
-                            "should be a CodeObject class, not a string. You can, however, "
-                            "send a string description of the target desired for the CodeObject "
-                            "under the keyword fallback_pref")
+            raise TypeError(
+                "codeobj_class argument given to code_object_class device method "
+                "should be a CodeObject class, not a string. You can, however, "
+                "send a string description of the target desired for the CodeObject "
+                "under the keyword fallback_pref"
+            )
         if codeobj_class is None:
             codeobj_class = prefs[fallback_pref]
             if isinstance(codeobj_class, str):
-                if codeobj_class == 'auto':
+                if codeobj_class == "auto":
                     return auto_target()
                 for target in codegen_targets:
                     if target.class_name == codeobj_class:
                         return target
                 # No target found
-                targets = ['auto'] + [target.class_name
-                                      for target in codegen_targets
-                                      if target.class_name]
-                raise ValueError(f"Unknown code generation target: {codeobj_class}, should be  one of {targets}")
+                targets = ["auto"] + [
+                    target.class_name for target in codegen_targets if target.class_name
+                ]
+                raise ValueError(
+                    f"Unknown code generation target: {codeobj_class}, should be  one"
+                    f" of {targets}"
+                )
         else:
             return codeobj_class
 
-    def code_object(self, owner, name, abstract_code, variables, template_name,
-                    variable_indices, codeobj_class=None,
-                    template_kwds=None, override_conditional_write=None,
-                    compiler_kwds=None):
+    def code_object(
+        self,
+        owner,
+        name,
+        abstract_code,
+        variables,
+        template_name,
+        variable_indices,
+        codeobj_class=None,
+        template_kwds=None,
+        override_conditional_write=None,
+        compiler_kwds=None,
+    ):
         if compiler_kwds is None:
             compiler_kwds = {}
         name = find_name(name)
         codeobj_class = self.code_object_class(codeobj_class)
         template = getattr(codeobj_class.templater, template_name)
         iterate_all = template.iterate_all
-        generator = codeobj_class.generator_class(variables=variables,
-                                                  variable_indices=variable_indices,
-                                                  owner=owner,
-                                                  iterate_all=iterate_all,
-                                                  codeobj_class=codeobj_class,
-                                                  override_conditional_write=override_conditional_write,
-                                                  allows_scalar_write=template.allows_scalar_write,
-                                                  name=name,
-                                                  template_name=template_name)
+        generator = codeobj_class.generator_class(
+            variables=variables,
+            variable_indices=variable_indices,
+            owner=owner,
+            iterate_all=iterate_all,
+            codeobj_class=codeobj_class,
+            override_conditional_write=override_conditional_write,
+            allows_scalar_write=template.allows_scalar_write,
+            name=name,
+            template_name=template_name,
+        )
         if template_kwds is None:
             template_kwds = dict()
         else:
             template_kwds = template_kwds.copy()
 
-        logger.diagnostic(f'{name} abstract code:\n{indent(code_representation(abstract_code))}')
+        logger.diagnostic(
+            f"{name} abstract code:\n{indent(code_representation(abstract_code))}"
+        )
 
-        scalar_code, vector_code, kwds = generator.translate(abstract_code,
-                                                             dtype=prefs['core.default_float_dtype'])
+        scalar_code, vector_code, kwds = generator.translate(
+            abstract_code, dtype=prefs["core.default_float_dtype"]
+        )
         # Add the array names as keywords as well
         for varname, var in variables.items():
             if isinstance(var, ArrayVariable):
                 pointer_name = generator.get_array_name(var)
                 if var.scalar:
-                    pointer_name += '[0]'
+                    pointer_name += "[0]"
                 template_kwds[varname] = pointer_name
-                if hasattr(var, 'resize'):
-                    dyn_array_name = generator.get_array_name(var,
-                                                              access_data=False)
+                if hasattr(var, "resize"):
+                    dyn_array_name = generator.get_array_name(var, access_data=False)
                     template_kwds[f"_dynamic_{varname}"] = dyn_array_name
 
-
         template_kwds.update(kwds)
-        logger.diagnostic(f'{name} snippet (scalar):\n{indent(code_representation(scalar_code))}')
-        logger.diagnostic(f'{name} snippet (vector):\n{indent(code_representation(vector_code))}')
+        logger.diagnostic(
+            f"{name} snippet (scalar):\n{indent(code_representation(scalar_code))}"
+        )
+        logger.diagnostic(
+            f"{name} snippet (vector):\n{indent(code_representation(vector_code))}"
+        )
 
-        code = template(scalar_code, vector_code,
-                        owner=owner, variables=variables, codeobj_name=name,
-                        variable_indices=variable_indices,
-                        get_array_name=generator.get_array_name,
-                        **template_kwds)
-        logger.diagnostic(f'{name} code:\n{indent(code_representation(code))}')
+        code = template(
+            scalar_code,
+            vector_code,
+            owner=owner,
+            variables=variables,
+            codeobj_name=name,
+            variable_indices=variable_indices,
+            get_array_name=generator.get_array_name,
+            **template_kwds,
+        )
+        logger.diagnostic(f"{name} code:\n{indent(code_representation(code))}")
 
-        codeobj = codeobj_class(owner, code, variables, variable_indices,
-                                template_name=template_name,
-                                template_source=template.template_source,
-                                name=name, compiler_kwds=compiler_kwds)
+        codeobj = codeobj_class(
+            owner,
+            code,
+            variables,
+            variable_indices,
+            template_name=template_name,
+            template_source=template.template_source,
+            name=name,
+            compiler_kwds=compiler_kwds,
+        )
         codeobj.compile()
         return codeobj
-    
+
     def activate(self, build_on_run=True, **kwargs):
         """
         Called when this device is set as the current device.
@@ -331,22 +375,23 @@ class Device(object):
         from brian2.core.clocks import Clock  # avoid import issues
 
         if self.defaultclock is None:
-            self.defaultclock = Clock(dt=0.1*ms, name='defaultclock')
+            self.defaultclock = Clock(dt=0.1 * ms, name="defaultclock")
         self._set_maximum_run_time(None)
         self.build_on_run = build_on_run
         self.build_options = dict(kwargs)
 
     def insert_device_code(self, slot, code):
         # Deprecated
-        raise AttributeError("The method 'insert_device_code' has been renamed "
-                             "to 'insert_code'.")
+        raise AttributeError(
+            "The method 'insert_device_code' has been renamed to 'insert_code'."
+        )
 
     def insert_code(self, slot, code):
         """
         Insert code directly into a given slot in the device. By default does nothing.
         """
         logger.warn(f"Ignoring device code, unknown slot: {slot}, code: {code}")
-        
+
     def build(self, **kwds):
         """
         For standalone projects, called when the project is ready to be built. Does nothing for runtime mode.
@@ -399,8 +444,9 @@ class Device(object):
             The state of the random number generator in a representation
             that can be passed as an argument to `.Device.set_random_state`.
         """
-        raise NotImplementedError("Device does not support getting the state "
-                                  "of the random number generator.")
+        raise NotImplementedError(
+            "Device does not support getting the state of the random number generator."
+        )
 
     def set_random_state(self, state):
         """
@@ -413,8 +459,9 @@ class Device(object):
             A random number generator state as provided by
             `Device.get_random_state`.
         """
-        raise NotImplementedError("Device does not support setting the state "
-                                  "of the random number generator.")
+        raise NotImplementedError(
+            "Device does not support setting the state of the random number generator."
+        )
 
 
 class RuntimeDevice(Device):
@@ -422,8 +469,9 @@ class RuntimeDevice(Device):
     The default device used in Brian, state variables are stored as numpy
     arrays in memory.
     """
+
     def __init__(self):
-        super(RuntimeDevice, self).__init__()
+        super().__init__()
         #: Mapping from `Variable` objects to numpy arrays (or `DynamicArray`
         #: objects). Arrays in this dictionary will disappear as soon as the
         #: last reference to the `Variable` object used as a key is gone
@@ -439,7 +487,7 @@ class RuntimeDevice(Device):
         # if no owner is set, this is a temporary object (e.g. the array
         # of indices when doing G.x[indices] = ...). The name is not
         # necessarily unique over several CodeObjects in this case.
-        owner_name = getattr(var.owner, 'name', 'temporary')
+        owner_name = getattr(var.owner, "name", "temporary")
 
         if isinstance(var, DynamicArrayVariable):
             if access_data:
@@ -465,7 +513,7 @@ class RuntimeDevice(Device):
 
     def get_value(self, var, access_data=True):
         if isinstance(var, DynamicArrayVariable) and access_data:
-                return self.arrays[var].data
+            return self.arrays[var].data
         else:
             return self.arrays[var]
 
@@ -482,8 +530,7 @@ class RuntimeDevice(Device):
         self.arrays[var][:] = 0
 
     def init_with_arange(self, var, start, dtype):
-        self.arrays[var][:] = np.arange(start, stop=var.get_len()+start,
-                                        dtype=dtype)
+        self.arrays[var][:] = np.arange(start, stop=var.get_len() + start, dtype=dtype)
 
     def fill_with_array(self, var, arr):
         self.arrays[var][:] = arr
@@ -492,10 +539,12 @@ class RuntimeDevice(Device):
         # Use the C++ version of the SpikeQueue when available
         try:
             from brian2.synapses.cythonspikequeue import SpikeQueue
-            logger.diagnostic('Using the C++ SpikeQueue', once=True)
+
+            logger.diagnostic("Using the C++ SpikeQueue", once=True)
         except ImportError:
             from brian2.synapses.spikequeue import SpikeQueue
-            logger.diagnostic('Using the Python SpikeQueue', once=True)
+
+            logger.diagnostic("Using the Python SpikeQueue", once=True)
 
         return SpikeQueue(source_start=source_start, source_end=source_end)
 
@@ -514,60 +563,71 @@ class RuntimeDevice(Device):
         self.randn_buffer_index[:] = 0
 
     def get_random_state(self):
-        return {'numpy_state': np.random.get_state(),
-                'rand_buffer_index': np.array(self.rand_buffer_index),
-                'rand_buffer': np.array(self.rand_buffer),
-                'randn_buffer_index': np.array(self.randn_buffer_index),
-                'randn_buffer': np.array(self.randn_buffer)
-                }
+        return {
+            "numpy_state": np.random.get_state(),
+            "rand_buffer_index": np.array(self.rand_buffer_index),
+            "rand_buffer": np.array(self.rand_buffer),
+            "randn_buffer_index": np.array(self.randn_buffer_index),
+            "randn_buffer": np.array(self.randn_buffer),
+        }
 
     def set_random_state(self, state):
-        np.random.set_state(state['numpy_state'])
-        self.rand_buffer_index[:] = state['rand_buffer_index']
-        self.rand_buffer[:] = state['rand_buffer']
-        self.randn_buffer_index[:] = state['randn_buffer_index']
-        self.randn_buffer[:] = state['randn_buffer']
+        np.random.set_state(state["numpy_state"])
+        self.rand_buffer_index[:] = state["rand_buffer_index"]
+        self.rand_buffer[:] = state["rand_buffer"]
+        self.randn_buffer_index[:] = state["randn_buffer_index"]
+        self.randn_buffer[:] = state["randn_buffer"]
 
 
-class Dummy(object):
+class Dummy:
     """
     Dummy object
     """
+
     def __getattr__(self, name):
         return Dummy()
+
     def __call__(self, *args, **kwds):
         return Dummy()
+
     def __enter__(self):
         return Dummy()
+
     def __exit__(self, type, value, traceback):
         pass
+
     def __getitem__(self, i):
         return Dummy()
+
     def __setitem__(self, i, val):
         pass
 
 
-class CurrentDeviceProxy(object):
+class CurrentDeviceProxy:
     """
     Method proxy for access to the currently active device
     """
+
     def __getattr__(self, name):
         if not hasattr(active_device, name):
             # We special case the name "shape" here, since some IDEs (e.g. The Python
             # console in PyDev and PyCharm) use the "shape" attribute to determine
             # whether an object is "array-like".
-            if name.startswith('_') or name == 'shape':
+            if name.startswith("_") or name == "shape":
                 # Do not fake private/magic attributes
-                raise AttributeError(f"Active device does not have an attribute "
-                                     f"'{name}'.")
+                raise AttributeError(
+                    f"Active device does not have an attribute '{name}'."
+                )
             else:
-                logger.warn(f"Active device does not have an attribute '{name}', "
-                            f"ignoring this.")
+                logger.warn(
+                    f"Active device does not have an attribute '{name}', ignoring this."
+                )
                 attr = Dummy()
         else:
             attr = getattr(active_device, name)
         return attr
 
+
 #: Proxy object to access methods of the current device
 device = CurrentDeviceProxy()
 #: The currently active device (set with `set_device`)
@@ -581,6 +641,7 @@ def get_device():
     global active_device
     return active_device
 
+
 #: A stack of previously set devices as a tuple with their options (see
 #: `set_device`): (device, build_on_run, build_options)
 previous_devices = []
@@ -606,11 +667,9 @@ def set_device(device, build_on_run=True, **kwargs):
     """
     global previous_devices
     if active_device is not None:
-        prev_build_on_run = getattr(active_device, 'build_on_run', True)
-        prev_build_options = getattr(active_device, 'build_options', {})
-        previous_devices.append((active_device,
-                                 prev_build_on_run,
-                                 prev_build_options))
+        prev_build_on_run = getattr(active_device, "build_on_run", True)
+        prev_build_options = getattr(active_device, "build_options", {})
+        previous_devices.append((active_device, prev_build_on_run, prev_build_options))
     _do_set_device(device, build_on_run, **kwargs)
 
 
@@ -710,10 +769,9 @@ def seed(seed=None):
     This function delegates the call to `Device.seed` of the current device.
     """
     if seed is not None and not isinstance(seed, numbers.Integral):
-        raise TypeError(f'Seed has to be None or an integer, was {type(seed)}')
+        raise TypeError(f"Seed has to be None or an integer, was {type(seed)}")
     get_device().seed(seed)
 
 
 runtime_device = RuntimeDevice()
-all_devices['runtime'] = runtime_device
-
+all_devices["runtime"] = runtime_device
