@@ -2,9 +2,8 @@
 Base package for all monitors, i.e. objects to record activity during a
 simulation run.
 """
+from .ratemonitor import *
 from .spikemonitor import *
 from .statemonitor import *
-from .ratemonitor import *
 
-__all__ = ['SpikeMonitor', 'EventMonitor', 'StateMonitor',
-           'PopulationRateMonitor']
+__all__ = ["SpikeMonitor", "EventMonitor", "StateMonitor", "PopulationRateMonitor"]
