@@ -3,32 +3,31 @@ Brian AST representation
 
 This is a standard Python AST representation with additional information added.
 """
-
-
 import ast
 import weakref
 
 import numpy
-from builtins import all as logical_all  # defensive programming against numpy import
 
-from brian2.parsing.rendering import NodeRenderer, get_node_value
+from brian2.parsing.rendering import get_node_value
 from brian2.utils.logger import get_logger
 
-__all__ = ['brian_ast', 'BrianASTRenderer', 'dtype_hierarchy']
+__all__ = ["brian_ast", "BrianASTRenderer", "dtype_hierarchy"]
 
 
 logger = get_logger(__name__)
 
 # This codifies the idea that operations involving e.g. boolean and integer will end up
 # as integer. In general the output type will be the max of the hierarchy values here.
-dtype_hierarchy = {'boolean': 0,
-                   'integer': 1,
-                   'float': 2,
-                   }
+dtype_hierarchy = {
+    "boolean": 0,
+    "integer": 1,
+    "float": 2,
+}
 # This is just so you can invert from number to string
 for tc, i in dict(dtype_hierarchy).items():
     dtype_hierarchy[i] = tc
 
+
 def is_boolean(value):
     return isinstance(value, bool)
 
@@ -46,26 +45,35 @@ def brian_dtype_from_value(value):
     Returns 'boolean', 'integer' or 'float'
     """
     if is_float(value):
-        return 'float'
+        return "float"
     elif is_integer(value):
-        return 'integer'
+        return "integer"
     elif is_boolean(value):
-        return 'boolean'
+        return "boolean"
     raise TypeError(f"Unknown dtype for value {str(value)}")
 
+
 # The following functions are called very often during the optimisation process
 # so we don't use numpy.issubdtype but instead a precalculated list of all
 # standard types
 
 bool_dtype = numpy.dtype(bool)
+
+
 def is_boolean_dtype(obj):
     return numpy.dtype(obj) is bool_dtype
 
-integer_dtypes = {numpy.dtype(c) for c in numpy.typecodes['AllInteger']}
+
+integer_dtypes = {numpy.dtype(c) for c in numpy.typecodes["AllInteger"]}
+
+
 def is_integer_dtype(obj):
     return numpy.dtype(obj) in integer_dtypes
 
-float_dtypes = {numpy.dtype(c) for c in numpy.typecodes['AllFloat']}
+
+float_dtypes = {numpy.dtype(c) for c in numpy.typecodes["AllFloat"]}
+
+
 def is_float_dtype(obj):
     return numpy.dtype(obj) in float_dtypes
 
@@ -75,11 +83,11 @@ def brian_dtype_from_dtype(dtype):
     Returns 'boolean', 'integer' or 'float'
     """
     if is_float_dtype(dtype):
-        return 'float'
+        return "float"
     elif is_integer_dtype(dtype):
-        return 'integer'
+        return "integer"
     elif is_boolean_dtype(dtype):
-        return 'boolean'
+        return "boolean"
     raise TypeError(f"Unknown dtype: {str(dtype)}")
 
 
@@ -108,15 +116,16 @@ def brian_ast(expr, variables):
     variables : dict
         The dictionary of `Variable` objects used in the expression.
     """
-    node = ast.parse(expr, mode='eval').body
+    node = ast.parse(expr, mode="eval").body
     renderer = BrianASTRenderer(variables)
     return renderer.render_node(node)
 
 
-class BrianASTRenderer(object):
+class BrianASTRenderer:
     """
     This class is modelled after `NodeRenderer` - see there for details.
     """
+
     def __init__(self, variables, copy_variables=True):
         if copy_variables:
             self.variables = variables.copy()
@@ -135,7 +144,7 @@ class BrianASTRenderer(object):
         if node.value is not True and node.value is not False:
             raise SyntaxError(f"Unknown NameConstant {str(node.value)}")
         # NameConstant only used for True and False and None, and we don't support None
-        node.dtype = 'boolean'
+        node.dtype = "boolean"
         node.scalar = True
         node.complexity = 0
         node.stateless = True
@@ -143,17 +152,17 @@ class BrianASTRenderer(object):
 
     def render_Name(self, node):
         node.complexity = 0
-        if node.id=='True' or node.id=='False':
-            node.dtype = 'boolean'
+        if node.id == "True" or node.id == "False":
+            node.dtype = "boolean"
             node.scalar = True
         elif node.id in self.variables:
             var = self.variables[node.id]
             dtype = var.dtype
             node.dtype = brian_dtype_from_dtype(dtype)
             node.scalar = var.scalar
-        else: # don't think we need to handle other names (pi, e, inf)?
-            node.dtype = 'float'
-            node.scalar = True # I think this assumption is OK, but not certain
+        else:  # don't think we need to handle other names (pi, e, inf)?
+            node.dtype = "float"
+            node.scalar = True  # I think this assumption is OK, but not certain
         node.stateless = True
         return node
 
@@ -173,9 +182,9 @@ class BrianASTRenderer(object):
     def render_Call(self, node):
         if len(node.keywords):
             raise ValueError("Keyword arguments not supported.")
-        elif getattr(node, 'starargs', None) is not None:
+        elif getattr(node, "starargs", None) is not None:
             raise ValueError("Variable number of arguments not supported")
-        elif getattr(node, 'kwargs', None) is not None:
+        elif getattr(node, "kwargs", None) is not None:
             raise ValueError("Keyword arguments not supported")
         args = []
         for subnode in node.args:
@@ -183,33 +192,37 @@ class BrianASTRenderer(object):
             subnode = self.render_node(subnode)
             args.append(subnode)
         node.args = args
-        node.dtype = 'float' # default dtype
+        node.dtype = "float"  # default dtype
         # Condition for scalarity of function call: stateless and arguments are scalar
         node.scalar = False
         if node.func.id in self.variables:
             funcvar = self.variables[node.func.id]
             # sometimes this attribute doesn't exist, if so assume it's not stateless
-            node.stateless = getattr(funcvar, 'stateless', False)
-            node.auto_vectorise = getattr(funcvar, 'auto_vectorise', False)
+            node.stateless = getattr(funcvar, "stateless", False)
+            node.auto_vectorise = getattr(funcvar, "auto_vectorise", False)
             if node.stateless and not node.auto_vectorise:
-                node.scalar = logical_all(subnode.scalar for subnode in node.args)
+                node.scalar = all(subnode.scalar for subnode in node.args)
             # check that argument types are valid
             node_arg_types = [subnode.dtype for subnode in node.args]
             for subnode, argtype in zip(node.args, funcvar._arg_types):
-                if argtype!='any' and argtype!=subnode.dtype:
-                    raise TypeError(f"Function '{node.func.id}' takes arguments with "
-                                    f"types {funcvar._arg_types} but "
-                                    f"received {node_arg_types}.")
+                if argtype != "any" and argtype != subnode.dtype:
+                    raise TypeError(
+                        f"Function '{node.func.id}' takes arguments with "
+                        f"types {funcvar._arg_types} but "
+                        f"received {node_arg_types}."
+                    )
             # compute return type
             return_type = funcvar._return_type
-            if return_type=='highest':
-                return_type = dtype_hierarchy[max(dtype_hierarchy[nat] for nat in node_arg_types)]
+            if return_type == "highest":
+                return_type = dtype_hierarchy[
+                    max(dtype_hierarchy[nat] for nat in node_arg_types)
+                ]
             node.dtype = return_type
         else:
             node.stateless = False
         # we leave node.func because it is an ast.Name object that doesn't have a dtype
         # TODO: variable complexity for function calls?
-        node.complexity = 20+sum(subnode.complexity for subnode in node.args)
+        node.complexity = 20 + sum(subnode.complexity for subnode in node.args)
         return node
 
     def render_BinOp(self, node):
@@ -219,13 +232,15 @@ class BrianASTRenderer(object):
         node.right = self.render_node(node.right)
         # TODO: we could capture some syntax errors here, e.g. bool+bool
         # captures, e.g. int+float->float
-        newdtype = dtype_hierarchy[max(dtype_hierarchy[subnode.dtype] for subnode in [node.left, node.right])]
-        if node.op.__class__.__name__ == 'Div':
+        newdtype = dtype_hierarchy[
+            max(dtype_hierarchy[subnode.dtype] for subnode in [node.left, node.right])
+        ]
+        if node.op.__class__.__name__ == "Div":
             # Division turns integers into floating point values
-            newdtype = 'float'
+            newdtype = "float"
         node.dtype = newdtype
         node.scalar = node.left.scalar and node.right.scalar
-        node.complexity = 1+node.left.complexity+node.right.complexity
+        node.complexity = 1 + node.left.complexity + node.right.complexity
         node.stateless = node.left.stateless and node.right.stateless
         return node
 
@@ -236,14 +251,13 @@ class BrianASTRenderer(object):
             subnode = self.render_node(subnode)
             values.append(subnode)
         node.values = values
-        node.dtype = 'boolean'
+        node.dtype = "boolean"
         for subnode in node.values:
-            if subnode.dtype!='boolean':
+            if subnode.dtype != "boolean":
                 raise TypeError("Boolean operator acting on non-booleans")
-        node.scalar = logical_all(subnode.scalar for subnode in node.values)
-        node.complexity = 1+sum(subnode.complexity for subnode in node.values)
-        node.stateless = logical_all(subnode.stateless
-                                     for subnode in node.values)
+        node.scalar = all(subnode.scalar for subnode in node.values)
+        node.complexity = 1 + sum(subnode.complexity for subnode in node.values)
+        node.stateless = all(subnode.stateless for subnode in node.values)
         return node
 
     def render_Compare(self, node):
@@ -254,21 +268,25 @@ class BrianASTRenderer(object):
             subnode = self.render_node(subnode)
             comparators.append(subnode)
         node.comparators = comparators
-        node.dtype = 'boolean'
-        comparators = [node.left]+node.comparators
-        node.scalar = logical_all(subnode.scalar for subnode in comparators)
-        node.complexity = 1+sum(subnode.complexity for subnode in comparators)
-        node.stateless = node.left.stateless and all(c.stateless
-                                                     for c in node.comparators)
+        node.dtype = "boolean"
+        comparators = [node.left] + node.comparators
+        node.scalar = all(subnode.scalar for subnode in comparators)
+        node.complexity = 1 + sum(subnode.complexity for subnode in comparators)
+        node.stateless = node.left.stateless and all(
+            c.stateless for c in node.comparators
+        )
         return node
 
     def render_UnaryOp(self, node):
         node.operand.parent = node
         node.operand = self.render_node(node.operand)
         node.dtype = node.operand.dtype
-        if node.dtype=='boolean' and node.op.__class__.__name__ != 'Not':
-            raise TypeError(f"Unary operator {node.op.__class__.__name__} does not apply to boolean types")
+        if node.dtype == "boolean" and node.op.__class__.__name__ != "Not":
+            raise TypeError(
+                f"Unary operator {node.op.__class__.__name__} does not apply to boolean"
+                " types"
+            )
         node.scalar = node.operand.scalar
-        node.complexity = 1+node.operand.complexity
+        node.complexity = 1 + node.operand.complexity
         node.stateless = node.operand.stateless
         return node
