@@ -3,19 +3,18 @@
 """
 
 
-
 import inspect
-import textwrap
-import re
 import pydoc
+import re
+import textwrap
 from warnings import warn
 
 from sphinx.pycode import ModuleAnalyzer
 
-class Reader(object):
-    """A line-based string reader.
 
-    """
+class Reader:
+    """A line-based string reader."""
+
     def __init__(self, data):
         """
         Parameters
@@ -27,7 +26,7 @@ class Reader(object):
         if isinstance(data, list):
             self._str = data
         else:
-            self._str = data.split('\n') # store string as list of lines
+            self._str = data.split("\n")  # store string as list of lines
 
         self.reset()
 
@@ -35,7 +34,7 @@ class Reader(object):
         return self._str[n]
 
     def reset(self):
-        self._l = 0 # current line nr
+        self._l = 0  # current line nr
 
     def read(self):
         if not self.eof():
@@ -43,11 +42,11 @@ class Reader(object):
             self._l += 1
             return out
         else:
-            return ''
+            return ""
 
     def seek_next_non_empty_line(self):
-        for l in self[self._l:]:
-            if l.strip():
+        for line in self[self._l :]:
+            if line.strip():
                 break
             else:
                 self._l += 1
@@ -59,56 +58,59 @@ class Reader(object):
         start = self._l
         for line in self[start:]:
             if condition_func(line):
-                return self[start:self._l]
+                return self[start : self._l]
             self._l += 1
             if self.eof():
-                return self[start:self._l+1]
+                return self[start : self._l + 1]
         return []
 
     def read_to_next_empty_line(self):
         self.seek_next_non_empty_line()
+
         def is_empty(line):
             return not line.strip()
+
         return self.read_to_condition(is_empty)
 
     def read_to_next_unindented_line(self):
         def is_unindented(line):
-            return (line.strip() and (len(line.lstrip()) == len(line)))
+            return line.strip() and (len(line.lstrip()) == len(line))
+
         return self.read_to_condition(is_unindented)
 
     def peek(self, n=0):
         if self._l + n < len(self._str):
             return self[self._l + n]
         else:
-            return ''
+            return ""
 
     def is_empty(self):
-        return not ''.join(self._str).strip()
+        return not "".join(self._str).strip()
 
 
-class NumpyDocString(object):
-    def __init__(self, docstring, config={}):
-        docstring = textwrap.dedent(docstring).split('\n')
+class NumpyDocString:
+    def __init__(self, docstring, config=None):
+        docstring = textwrap.dedent(docstring).split("\n")
 
         self._doc = Reader(docstring)
         self._parsed_data = {
-            'Signature': '',
-            'Summary': [''],
-            'Extended Summary': [],
-            'Parameters': [],
-            'Returns': [],
-            'Raises': [],
-            'Warns': [],
-            'Other Parameters': [],
-            'Attributes': [],
-            'Methods': [],
-            'See Also': [],
-            'Notes': [],
-            'Warnings': [],
-            'References': '',
-            'Examples': '',
-            'index': {}
-            }
+            "Signature": "",
+            "Summary": [""],
+            "Extended Summary": [],
+            "Parameters": [],
+            "Returns": [],
+            "Raises": [],
+            "Warns": [],
+            "Other Parameters": [],
+            "Attributes": [],
+            "Methods": [],
+            "See Also": [],
+            "Notes": [],
+            "Warnings": [],
+            "References": "",
+            "Examples": "",
+            "index": {},
+        }
 
         self._parse()
 
@@ -129,31 +131,32 @@ class NumpyDocString(object):
 
         l1 = self._doc.peek().strip()  # e.g. Parameters
 
-        if l1.startswith('.. index::'):
+        if l1.startswith(".. index::"):
             return True
 
-        l2 = self._doc.peek(1).strip() #    ---------- or ==========
-        return l2.startswith('-'*len(l1)) or l2.startswith('='*len(l1))
+        l2 = self._doc.peek(1).strip()  # ---------- or ==========
+        return l2.startswith("-" * len(l1)) or l2.startswith("=" * len(l1))
 
     def _strip(self, doc):
-        i = 0
-        j = 0
+        start = stop = 0
         for i, line in enumerate(doc):
             if line.strip():
+                start = i
                 break
 
-        for j, line in enumerate(doc[::-1]):
+        for i, line in enumerate(doc[::-1]):
             if line.strip():
+                stop = i
                 break
 
-        return doc[i:len(doc)-j]
+        return doc[start:-stop]
 
     def _read_to_next_section(self):
         section = self._doc.read_to_next_empty_line()
 
         while not self._is_at_section() and not self._doc.eof():
-            if not self._doc.peek(-1).strip(): # previous line was empty
-                section += ['']
+            if not self._doc.peek(-1).strip():  # previous line was empty
+                section += [""]
 
             section += self._doc.read_to_next_empty_line()
 
@@ -164,7 +167,7 @@ class NumpyDocString(object):
             data = self._read_to_next_section()
             name = data[0].strip()
 
-            if name.startswith('..'): # index section
+            if name.startswith(".."):  # index section
                 yield name, data[1:]
             elif len(data) < 2:
                 yield StopIteration
@@ -176,10 +179,10 @@ class NumpyDocString(object):
         params = []
         while not r.eof():
             header = r.read().strip()
-            if ' : ' in header:
-                arg_name, arg_type = header.split(' : ')[:2]
+            if " : " in header:
+                arg_name, arg_type = header.split(" : ")[:2]
             else:
-                arg_name, arg_type = header, ''
+                arg_name, arg_type = header, ""
 
             desc = r.read_to_next_unindented_line()
             desc = dedent_lines(desc)
@@ -188,9 +191,12 @@ class NumpyDocString(object):
 
         return params
 
+    _name_rgx = re.compile(
+        r"^\s*(:(?P<role>\w+):`(?P<name>[a-zA-Z0-9_.-]+)`|"
+        r" (?P<name2>[a-zA-Z0-9_.-]+))\s*",
+        re.X,
+    )
 
-    _name_rgx = re.compile(r"^\s*(:(?P<role>\w+):`(?P<name>[a-zA-Z0-9_.-]+)`|"
-                           r" (?P<name2>[a-zA-Z0-9_.-]+))\s*", re.X)
     def _parse_see_also(self, content):
         """
         func_name : Descriptive text
@@ -227,17 +233,17 @@ class NumpyDocString(object):
                 continue
 
             m = self._name_rgx.match(line)
-            if m and line[m.end():].strip().startswith(':'):
+            if m and line[m.end() :].strip().startswith(":"):
                 push_item(current_func, rest)
-                current_func, line = line[:m.end()], line[m.end():]
-                rest = [line.split(':', 1)[1].strip()]
+                current_func, line = line[: m.end()], line[m.end() :]
+                rest = [line.split(":", 1)[1].strip()]
                 if not rest[0]:
                     rest = []
-            elif not line.startswith(' '):
+            elif not line.startswith(" "):
                 push_item(current_func, rest)
                 current_func = None
-                if ',' in line:
-                    for func in line.split(','):
+                if "," in line:
+                    for func in line.split(","):
                         if func.strip():
                             push_item(func, [])
                 elif line.strip():
@@ -253,17 +259,18 @@ class NumpyDocString(object):
            :refguide: something, else, and more
 
         """
+
         def strip_each_in(lst):
             return [s.strip() for s in lst]
 
         out = {}
-        section = section.split('::')
+        section = section.split("::")
         if len(section) > 1:
-            out['default'] = strip_each_in(section[1].split(','))[0]
+            out["default"] = strip_each_in(section[1].split(","))[0]
         for line in content:
-            line = line.split(':')
+            line = line.split(":")
             if len(line) > 2:
-                out[line[1]] = strip_each_in(line[2].split(','))
+                out[line[1]] = strip_each_in(line[2].split(","))
         return out
 
     def _parse_summary(self):
@@ -273,59 +280,66 @@ class NumpyDocString(object):
 
         summary = self._doc.read_to_next_empty_line()
         summary_str = " ".join([s.strip() for s in summary]).strip()
-        if re.compile(r'^([\w., ]+=)?\s*[\w.]+\(.*\)$').match(summary_str):
-            self['Signature'] = summary_str
+        if re.compile(r"^([\w., ]+=)?\s*[\w.]+\(.*\)$").match(summary_str):
+            self["Signature"] = summary_str
             if not self._is_at_section():
-                self['Summary'] = self._doc.read_to_next_empty_line()
+                self["Summary"] = self._doc.read_to_next_empty_line()
         else:
-            self['Summary'] = summary
+            self["Summary"] = summary
 
         if not self._is_at_section():
-            self['Extended Summary'] = self._read_to_next_section()
+            self["Extended Summary"] = self._read_to_next_section()
 
     def _parse(self):
         self._doc.reset()
         self._parse_summary()
 
-        for (section, content) in self._read_sections():
-            if not section.startswith('..'):
-                section = ' '.join([s.capitalize() for s in section.split(' ')])
-            if section in ('Parameters', 'Returns', 'Raises', 'Warns',
-                           'Other Parameters', 'Attributes', 'Methods'):
+        for section, content in self._read_sections():
+            if not section.startswith(".."):
+                section = " ".join([s.capitalize() for s in section.split(" ")])
+            if section in (
+                "Parameters",
+                "Returns",
+                "Raises",
+                "Warns",
+                "Other Parameters",
+                "Attributes",
+                "Methods",
+            ):
                 self[section] = self._parse_param_list(content)
-            elif section.startswith('.. index::'):
-                self['index'] = self._parse_index(section, content)
-            elif section == 'See Also':
-                self['See Also'] = self._parse_see_also(content)
+            elif section.startswith(".. index::"):
+                self["index"] = self._parse_index(section, content)
+            elif section == "See Also":
+                self["See Also"] = self._parse_see_also(content)
             else:
                 self[section] = content
 
     # string conversion routines
 
-    def _str_header(self, name, symbol='-'):
-        return [name, len(name)*symbol]
+    def _str_header(self, name, symbol="-"):
+        return [name, len(name) * symbol]
 
     def _str_indent(self, doc, indent=4):
         out = []
         for line in doc:
-            out += [' '*indent + line]
+            out += [" " * indent + line]
         return out
 
     def _str_signature(self):
-        if self['Signature']:
-            return [self['Signature'].replace('*', r'\*')] + ['']
+        if self["Signature"]:
+            return [self["Signature"].replace("*", r"\*")] + [""]
         else:
-            return ['']
+            return [""]
 
     def _str_summary(self):
-        if self['Summary']:
-            return self['Summary'] + ['']
+        if self["Summary"]:
+            return self["Summary"] + [""]
         else:
             return []
 
     def _str_extended_summary(self):
-        if self['Extended Summary']:
-            return self['Extended Summary'] + ['']
+        if self["Extended Summary"]:
+            return self["Extended Summary"] + [""]
         else:
             return []
 
@@ -334,9 +348,9 @@ class NumpyDocString(object):
         if self[name]:
             out += self._str_header(name)
             for param, param_type, desc in self[name]:
-                out += [f'{param} : {param_type}']
+                out += [f"{param} : {param_type}"]
                 out += self._str_indent(desc)
-            out += ['']
+            out += [""]
         return out
 
     def _str_section(self, name):
@@ -344,137 +358,144 @@ class NumpyDocString(object):
         if self[name]:
             out += self._str_header(name)
             out += self[name]
-            out += ['']
+            out += [""]
         return out
 
     def _str_see_also(self, func_role):
-        if not self['See Also']:
+        if not self["See Also"]:
             return []
         out = []
         out += self._str_header("See Also")
         last_had_desc = True
-        for func, desc, role in self['See Also']:
+        for func, desc, role in self["See Also"]:
             if role:
-                link = f':{role}:`{func}`'
+                link = f":{role}:`{func}`"
             elif func_role:
-                link = f':{func_role}:`{func}`'
+                link = f":{func_role}:`{func}`"
             else:
                 link = f"`{func}`_"
             if desc or last_had_desc:
-                out += ['']
+                out += [""]
                 out += [link]
             else:
                 out[-1] += f", {link}"
             if desc:
-                out += self._str_indent([' '.join(desc)])
+                out += self._str_indent([" ".join(desc)])
                 last_had_desc = True
             else:
                 last_had_desc = False
-        out += ['']
+        out += [""]
         return out
 
     def _str_index(self):
-        idx = self['index']
+        idx = self["index"]
         out = []
         out += [f".. index:: {idx.get('default', '')}"]
         for section, references in idx.items():
-            if section == 'default':
+            if section == "default":
                 continue
             out += [f"   :{section}: {', '.join(references)}"]
         return out
 
-    def __str__(self, func_role=''):
+    def __str__(self, func_role=""):
         out = []
         out += self._str_signature()
         out += self._str_summary()
         out += self._str_extended_summary()
-        for param_list in ('Parameters', 'Returns', 'Other Parameters',
-                           'Raises', 'Warns'):
+        for param_list in (
+            "Parameters",
+            "Returns",
+            "Other Parameters",
+            "Raises",
+            "Warns",
+        ):
             out += self._str_param_list(param_list)
-        out += self._str_section('Warnings')
+        out += self._str_section("Warnings")
         out += self._str_see_also(func_role)
-        for s in ('Notes', 'References', 'Examples'):
+        for s in ("Notes", "References", "Examples"):
             out += self._str_section(s)
-        for param_list in ('Attributes', 'Methods'):
+        for param_list in ("Attributes", "Methods"):
             out += self._str_param_list(param_list)
         out += self._str_index()
-        return '\n'.join(out)
+        return "\n".join(out)
 
 
 def indent(str, indent=4):
-    indent_str = ' '*indent
+    indent_str = " " * indent
     if str is None:
         return indent_str
-    lines = str.split('\n')
-    return '\n'.join(indent_str + l for l in lines)
+    lines = str.split("\n")
+    return "\n".join(indent_str + line for line in lines)
+
 
 def dedent_lines(lines):
     """Deindent a list of lines maximally"""
     return textwrap.dedent("\n".join(lines)).split("\n")
 
-def header(text, style='-'):
+
+def header(text, style="-"):
     return f"{text}\n{style * len(text)}\n"
 
 
 class FunctionDoc(NumpyDocString):
-    def __init__(self, func, role='func', doc=None, config={}):
+    def __init__(self, func, role="func", doc=None, config=None):
         self._f = func
-        self._role = role # e.g. "func" or "meth"
+        self._role = role  # e.g. "func" or "meth"
 
         if doc is None:
             if func is None:
                 raise ValueError("No function or docstring given")
-            doc = inspect.getdoc(func) or ''
+            doc = inspect.getdoc(func) or ""
         NumpyDocString.__init__(self, doc)
 
-        if not self['Signature'] and func is not None:
+        if not self["Signature"] and func is not None:
             func, func_name = self.get_func()
             try:
                 # try to read signature
                 argspec = str(inspect.signature(func))
-                argspec = argspec.replace('*', r'\*')
-                signature = f'{func_name}{argspec}'
+                argspec = argspec.replace("*", r"\*")
+                signature = f"{func_name}{argspec}"
             except (TypeError, ValueError):
-                signature = f'{func_name}()'
-            self['Signature'] = signature
+                signature = f"{func_name}()"
+            self["Signature"] = signature
 
     def get_func(self):
-        func_name = getattr(self._f, '__name__', self.__class__.__name__)
+        func_name = getattr(self._f, "__name__", self.__class__.__name__)
         if inspect.isclass(self._f):
-            func = getattr(self._f, '__call__', self._f.__init__)
+            if callable(self._f):
+                func = self._f.__call__
+            else:
+                func = self._f.__init__
         else:
             func = self._f
         return func, func_name
 
     def __str__(self):
-        out = ''
+        out = ""
 
         _, func_name = self.get_func()
 
-        roles = {'func': 'function',
-                 'meth': 'method'}
+        roles = {"func": "function", "meth": "method"}
 
         if self._role:
             if self._role not in roles:
                 print(f"Warning: invalid role {self._role}")
             out += f".. {roles.get(self._role, '')}:: {func_name}\n    \n\n"
 
-        out += super(FunctionDoc, self).__str__(func_role=self._role)
+        out += super().__str__(func_role=self._role)
         return out
 
 
 class ClassDoc(NumpyDocString):
+    extra_public_methods = ["__call__"]
 
-    extra_public_methods = ['__call__']
-
-    def __init__(self, cls, doc=None, modulename='', func_doc=FunctionDoc,
-                 config={}):
+    def __init__(self, cls, doc=None, modulename="", func_doc=FunctionDoc, config=None):
         if not inspect.isclass(cls) and cls is not None:
             raise ValueError(f"Expected a class or None, but got {cls!r}")
         self._cls = cls
 
-        if modulename and not modulename.endswith('.'):
-            modulename += '.'
+        if modulename and not modulename.endswith("."):
+            modulename += "."
         self._mod = modulename
 
         if doc is None:
@@ -484,22 +505,26 @@ class ClassDoc(NumpyDocString):
 
         NumpyDocString.__init__(self, doc)
 
-        if not self['Methods']:
-            self['Methods'] = [(name, '', '')
-                               for name in sorted(self.methods)]
-        if not self['Attributes']:
-            self['Attributes'] = [(name, '', '')
-                                  for name in sorted(self.properties)]
+        if not self["Methods"]:
+            self["Methods"] = [(name, "", "") for name in sorted(self.methods)]
+        if not self["Attributes"]:
+            self["Attributes"] = [(name, "", "") for name in sorted(self.properties)]
 
     @property
     def methods(self):
         if self._cls is None:
-            return [] 
-        methods = [name for name, func in getattr(self._cls, '__dict__').items()
-                   if ((not name.startswith('_')
-                        or name in self.extra_public_methods)
-                       and ((callable(func) and not isinstance(func, type)) or
-                            inspect.ismethoddescriptor(func)))]
+            return []
+        methods = [
+            name
+            for name, func in self._cls.__dict__.items()
+            if (
+                (not name.startswith("_") or name in self.extra_public_methods)
+                and (
+                    (callable(func) and not isinstance(func, type))
+                    or inspect.ismethoddescriptor(func)
+                )
+            )
+        ]
         return methods
 
     @property
@@ -507,11 +532,16 @@ class ClassDoc(NumpyDocString):
         if self._cls is None:
             return []
         analyzer = ModuleAnalyzer.for_module(self._cls.__module__)
-        instance_members = {attr_name for (class_name, attr_name) in
-                            analyzer.find_attr_docs().keys()
-                            if class_name == self._cls.__name__}
-        class_members = {name for name, func in getattr(self._cls, '__dict__').items()
-                         if not name.startswith('_') and (func is None or
-                                                          inspect.isdatadescriptor(func))}
+        instance_members = {
+            attr_name
+            for (class_name, attr_name) in analyzer.find_attr_docs().keys()
+            if class_name == self._cls.__name__
+        }
+        class_members = {
+            name
+            for name, func in self._cls.__dict__.items()
+            if not name.startswith("_")
+            and (func is None or inspect.isdatadescriptor(func))
+        }
 
         return instance_members | class_members
