@@ -4,12 +4,12 @@ of parsing, generation of state update code, etc.). Provides the `cached`
 decorator.
 """
 
-import functools
 import collections
+import functools
 from collections.abc import Mapping
 
 
-class CacheKey(object):
+class CacheKey:
     """
     Mixin class for objects that will be used as keys for caching (e.g.
     `Variable` objects) and have to define a certain "identity" with respect
@@ -26,6 +26,7 @@ class CacheKey(object):
     all attributes that were not excluded in such a way; this tuple will be used
     as the key for caching purposes.
     """
+
     #: Set of attributes that should not be considered for caching of state
     #: update code, etc.
     _cache_irrelevant_attributes = set()
@@ -34,20 +35,24 @@ class CacheKey(object):
     def _state_tuple(self):
         """A tuple with this object's attribute values, defining its identity
         for caching purposes. See `CacheKey` for details."""
-        return tuple(value for key, value in sorted(self.__dict__.items())
-                     if key not in self._cache_irrelevant_attributes)
+        return tuple(
+            value
+            for key, value in sorted(self.__dict__.items())
+            if key not in self._cache_irrelevant_attributes
+        )
 
 
-class _CacheStatistics(object):
+class _CacheStatistics:
     """
     Helper class to store cache statistics
     """
+
     def __init__(self):
         self.hits = 0
         self.misses = 0
 
     def __repr__(self):
-        return f'<Cache statistics: {int(self.hits)} hits, {int(self.misses)} misses>'
+        return f"<Cache statistics: {int(self.hits)} hits, {int(self.misses)} misses>"
 
 
 def cached(func):
@@ -84,9 +89,10 @@ def cached(func):
     @functools.wraps(func)
     def cached_func(*args, **kwds):
         try:
-            cache_key = tuple([_hashable(arg) for arg in args] +
-                              [(key, _hashable(value))
-                               for key, value in sorted(kwds.items())])
+            cache_key = tuple(
+                [_hashable(arg) for arg in args]
+                + [(key, _hashable(value)) for key, value in sorted(kwds.items())]
+            )
         except TypeError:
             # If we cannot handle a type here, that most likely means that the
             # user provided an argument of a type we don't handle. This will
@@ -105,26 +111,30 @@ def cached(func):
 
 
 _of_type_cache = collections.defaultdict(set)
+
+
 def _of_type(obj_type, check_type):
     if (obj_type, check_type) not in _of_type_cache:
         _of_type_cache[(obj_type, check_type)] = issubclass(obj_type, check_type)
     return _of_type_cache[(obj_type, check_type)]
 
+
 def _hashable(obj):
     """Helper function to make a few data structures hashable (e.g. a
     dictionary gets converted to a frozenset). The function is specifically
     tailored to our use case and not meant to be generally useful."""
-    if hasattr(obj, '_state_tuple'):
+    if hasattr(obj, "_state_tuple"):
         return _hashable(obj._state_tuple)
     obj_type = type(obj)
     if _of_type(obj_type, Mapping):
-        return frozenset((_hashable(key), _hashable(value))
-                         for key, value in obj.items())
+        return frozenset(
+            (_hashable(key), _hashable(value)) for key, value in obj.items()
+        )
     elif _of_type(obj_type, set):
         return frozenset(_hashable(el) for el in obj)
     elif _of_type(obj_type, tuple) or _of_type(obj_type, list):
         return tuple(_hashable(el) for el in obj)
-    if hasattr(obj, 'dim') and getattr(obj, 'shape', None) == ():
+    if hasattr(obj, "dim") and getattr(obj, "shape", None) == ():
         # Scalar Quantity object
         return float(obj), obj.dim
     else:
@@ -133,4 +143,4 @@ def _hashable(obj):
             hash(obj)
             return obj
         except TypeError:
-            raise TypeError(f'Do not know how to handle object of type {type(obj)}')
+            raise TypeError(f"Do not know how to handle object of type {type(obj)}")
