@@ -1,4 +1,3 @@
-
 """
 Utility functions to get information about the environment Brian is running in.
 """
@@ -8,10 +7,10 @@ import builtins
 def running_from_ipython():
     """
     Check whether we are currently running under ipython.
-    
+
     Returns
     -------
     ipython : bool
         Whether running under ipython or not.
     """
-    return getattr(builtins, '__IPYTHON__', False)
+    return getattr(builtins, "__IPYTHON__", False)
