@@ -34,6 +34,7 @@ import logging
 import os
 import threading
 import time
+
 try:
     import warnings
 except ImportError:
@@ -66,13 +67,15 @@ __all__ = [
     "WindowsFileLock",
     "UnixFileLock",
     "SoftFileLock",
-    "FileLock"
+    "FileLock",
 ]
 
 __version__ = "3.0.12"
 
 
 _logger = None
+
+
 def logger():
     """Returns the logger instance used in this module."""
     global _logger
@@ -89,8 +92,7 @@ class Timeout(TimeoutError):
     """
 
     def __init__(self, lock_file):
-        """
-        """
+        """ """
         #: The path of the file lock.
         self.lock_file = lock_file
         return None
@@ -111,8 +113,7 @@ class Timeout(TimeoutError):
 # automatically.
 #
 # :seealso: issue #37 (memory leak)
-class _Acquire_ReturnProxy(object):
-
+class _Acquire_ReturnProxy:
     def __init__(self, lock):
         self.lock = lock
         return None
@@ -125,14 +126,13 @@ class _Acquire_ReturnProxy(object):
         return None
 
 
-class BaseFileLock(object):
+class BaseFileLock:
     """
     Implements the base class of a file lock.
     """
 
-    def __init__(self, lock_file, timeout = -1):
-        """
-        """
+    def __init__(self, lock_file, timeout=-1):
+        """ """
         # The path to the lock file.
         self._lock_file = lock_file
 
@@ -179,8 +179,7 @@ class BaseFileLock(object):
 
     @timeout.setter
     def timeout(self, value):
-        """
-        """
+        """ """
         self._timeout = float(value)
         return None
 
@@ -267,30 +266,40 @@ class BaseFileLock(object):
                 with self._thread_lock:
                     if not self.is_locked:
                         # Level 5 = DIAGNOSTIC
-                        logger().log(5, 'Attempting to acquire lock %s on %s', lock_id, lock_filename)
+                        logger().log(
+                            5,
+                            "Attempting to acquire lock %s on %s",
+                            lock_id,
+                            lock_filename,
+                        )
                         self._acquire()
 
                 if self.is_locked:
-                    logger().debug('Lock %s acquired on %s', lock_id, lock_filename)
+                    logger().debug("Lock %s acquired on %s", lock_id, lock_filename)
                     break
                 elif timeout >= 0 and time.time() - start_time > timeout:
-                    logger().debug('Timeout on acquiring lock %s on %s', lock_id, lock_filename)
+                    logger().debug(
+                        "Timeout on acquiring lock %s on %s", lock_id, lock_filename
+                    )
                     raise Timeout(self._lock_file)
                 else:
-                    logger().log(5,
-                        'Lock %s not acquired on %s, waiting %s seconds ...',
-                        lock_id, lock_filename, poll_intervall
+                    logger().log(
+                        5,
+                        "Lock %s not acquired on %s, waiting %s seconds ...",
+                        lock_id,
+                        lock_filename,
+                        poll_intervall,
                     )
                     time.sleep(poll_intervall)
-        except:
+        except BaseException:
             # Something did go wrong, so decrement the counter.
             with self._thread_lock:
                 self._lock_counter = max(0, self._lock_counter - 1)
 
             raise
-        return _Acquire_ReturnProxy(lock = self)
+        return _Acquire_ReturnProxy(lock=self)
 
-    def release(self, force = False):
+    def release(self, force=False):
         """
         Releases the file lock.
 
@@ -304,7 +313,6 @@ class BaseFileLock(object):
             every case.
         """
         with self._thread_lock:
-
             if self.is_locked:
                 self._lock_counter -= 1
 
@@ -312,10 +320,12 @@ class BaseFileLock(object):
                     lock_id = id(self)
                     lock_filename = self._lock_file
 
-                    logger().log(5, 'Attempting to release lock %s on %s', lock_id, lock_filename)
+                    logger().log(
+                        5, "Attempting to release lock %s on %s", lock_id, lock_filename
+                    )
                     self._release()
                     self._lock_counter = 0
-                    logger().debug('Lock %s released on %s', lock_id, lock_filename)
+                    logger().debug("Lock %s released on %s", lock_id, lock_filename)
 
         return None
 
@@ -328,13 +338,14 @@ class BaseFileLock(object):
         return None
 
     def __del__(self):
-        self.release(force = True)
+        self.release(force=True)
         return None
 
 
 # Windows locking mechanism
 # ~~~~~~~~~~~~~~~~~~~~~~~~~
 
+
 class WindowsFileLock(BaseFileLock):
     """
     Uses the :func:`msvcrt.locking` function to hard lock the lock file on
@@ -351,7 +362,7 @@ class WindowsFileLock(BaseFileLock):
         else:
             try:
                 msvcrt.locking(fd, msvcrt.LK_NBLCK, 1)
-            except (IOError, OSError):
+            except OSError:
                 os.close(fd)
             else:
                 self._lock_file_fd = fd
@@ -371,9 +382,11 @@ class WindowsFileLock(BaseFileLock):
             pass
         return None
 
+
 # Unix locking mechanism
 # ~~~~~~~~~~~~~~~~~~~~~~
 
+
 class UnixFileLock(BaseFileLock):
     """
     Uses the :func:`fcntl.flock` to hard lock the lock file on unix systems.
@@ -385,7 +398,7 @@ class UnixFileLock(BaseFileLock):
 
         try:
             fcntl.flock(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
-        except (IOError, OSError):
+        except OSError:
             os.close(fd)
         else:
             self._lock_file_fd = fd
@@ -402,9 +415,11 @@ class UnixFileLock(BaseFileLock):
         os.close(fd)
         return None
 
+
 # Soft lock
 # ~~~~~~~~~
 
+
 class SoftFileLock(BaseFileLock):
     """
     Simply watches the existence of the lock file.
@@ -414,7 +429,7 @@ class SoftFileLock(BaseFileLock):
         open_mode = os.O_WRONLY | os.O_CREAT | os.O_EXCL | os.O_TRUNC
         try:
             fd = os.open(self._lock_file, open_mode)
-        except (IOError, OSError):
+        except OSError:
             pass
         else:
             self._lock_file_fd = fd
@@ -448,4 +463,4 @@ else:
     FileLock = SoftFileLock
 
     if warnings is not None:
-        warnings.warn("only soft file lock is available")
+        warnings.warn("only soft file lock is available", stacklevel=1)
