@@ -5,8 +5,12 @@ File system tools
 
 import os
 
-__all__ = ['ensure_directory', 'ensure_directory_of_file', 'in_directory',
-           'copy_directory']
+__all__ = [
+    "ensure_directory",
+    "ensure_directory_of_file",
+    "in_directory",
+    "copy_directory",
+]
 
 
 def ensure_directory_of_file(f):
@@ -29,24 +33,27 @@ def ensure_directory(d):
     return d
 
 
-class in_directory(object):
+class in_directory:
     """
     Safely temporarily work in a subdirectory
-    
+
     Usage::
-    
+
         with in_directory(directory):
             ... do stuff here
-            
+
     Guarantees that the code in the with block will be executed in directory,
     and that after the block is completed we return to the original directory.
     """
+
     def __init__(self, new_dir):
         self.orig_dir = os.getcwd()
         self.new_dir = new_dir
+
     def __enter__(self):
         os.chdir(self.new_dir)
-    def __exit__(self,*exc_info):
+
+    def __exit__(self, *exc_info):
         os.chdir(self.orig_dir)
 
 
@@ -55,11 +62,11 @@ def copy_directory(source, target):
     Copies directory source to target.
     """
     relnames = []
-    sourcebase = os.path.normpath(source)+os.path.sep
-    for root, dirnames, filenames in os.walk(source):
+    sourcebase = os.path.normpath(source) + os.path.sep
+    for root, _, filenames in os.walk(source):
         for filename in filenames:
             fullname = os.path.normpath(os.path.join(root, filename))
-            relname = fullname.replace(sourcebase, '')
+            relname = fullname.replace(sourcebase, "")
             relnames.append(relname)
             tgtname = os.path.join(target, relname)
             ensure_directory_of_file(tgtname)
@@ -69,6 +76,6 @@ def copy_directory(source, target):
                 with open(tgtname) as f:
                     if f.read() == contents:
                         continue
-            with open(tgtname, 'w') as f:
+            with open(tgtname, "w") as f:
                 f.write(contents)
     return relnames
